"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendMultipleInstructions = exports.sendInstructions = void 0;
const web3_js_1 = require("@solana/web3.js");
const anchorError_1 = require("./anchorError");
function promiseAllInOrder(it) {
    return __awaiter(this, void 0, void 0, function* () {
        let ret = [];
        for (const i of it) {
            ret.push(yield i());
        }
        return ret;
    });
}
function sendInstructions(idlErrors, provider, instructions, signers, payer = provider.wallet.publicKey, commitment = "confirmed") {
    return __awaiter(this, void 0, void 0, function* () {
        const tx = new web3_js_1.Transaction();
        tx.feePayer = payer || provider.wallet.publicKey;
        tx.add(...instructions);
        try {
            return yield provider.send(tx, signers, {
                commitment,
                preflightCommitment: commitment
            });
        }
        catch (e) {
            console.error(e);
            const wrappedE = anchorError_1.ProgramError.parse(e, idlErrors);
            throw wrappedE == null ? e : wrappedE;
        }
    });
}
exports.sendInstructions = sendInstructions;
function truthy(value) {
    return !!value;
}
function sendMultipleInstructions(idlErrors, provider, instructionGroups, signerGroups, payer, finality = "confirmed") {
    return __awaiter(this, void 0, void 0, function* () {
        const recentBlockhash = (yield provider.connection.getRecentBlockhash('confirmed')).blockhash;
        const txns = instructionGroups.map((instructions, index) => {
            const signers = signerGroups[index];
            if (instructions.length > 0) {
                const tx = new web3_js_1.Transaction({
                    feePayer: payer || provider.wallet.publicKey,
                    recentBlockhash
                });
                tx.add(...instructions);
                if (signers.length > 0) {
                    tx.partialSign(...signers);
                }
                return tx;
            }
        }).filter(truthy);
        const txnsSigned = (yield provider.wallet.signAllTransactions(txns)).map(tx => tx.serialize());
        console.log("Sending multiple transactions...");
        try {
            return yield promiseAllInOrder(txnsSigned.map((txn) => () => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                const txid = yield provider.connection.sendRawTransaction(txn, {
                    skipPreflight: true
                });
                const result = yield provider.connection.confirmTransaction(txid, finality);
                if (result.value.err) {
                    const tx = yield provider.connection.getTransaction(txid, {
                        commitment: finality
                    });
                    console.error((_b = (_a = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _a === void 0 ? void 0 : _a.logMessages) === null || _b === void 0 ? void 0 : _b.join("\n"));
                    throw result.value.err;
                }
                return txid;
            })));
        }
        catch (e) {
            console.error(e);
            const wrappedE = anchorError_1.ProgramError.parse(e, idlErrors);
            throw wrappedE == null ? e : wrappedE;
        }
    });
}
exports.sendMultipleInstructions = sendMultipleInstructions;
//# sourceMappingURL=transaction.js.map